package com.huiju.welcome.controller;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.huiju.welcome.common.Action;
import com.huiju.welcome.common.WebSocketServer;
import com.huiju.welcome.common.base.ResponseBean;
import com.huiju.welcome.model.*;
import com.huiju.welcome.service.*;
import com.huiju.welcome.utils.AliOSSUtils;
import com.huiju.welcome.utils.StatusUtils;
import com.huiju.welcome.utils.XFYun;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.apache.http.HttpStatus;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;

import javax.annotation.PostConstruct;
import javax.servlet.http.HttpServletRequest;
import javax.websocket.server.PathParam;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.channels.FileChannel;
import java.time.Duration;
import java.time.LocalDateTime;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author FXF
 * @date 2019-03-13
 */
@RestController
@RequestMapping("/")
@Transactional(rollbackFor = Exception.class)
public class CameraController {
	private Logger log = LoggerFactory.getLogger(XFYun.class);

	private final static int NEW_PERSON = 1;
	
	@Autowired
	private ITaPersonService taPersonService;

	@Autowired
	private ITaSnapshotPersonService taSnapshotPersonService;
	
	@Autowired
	private ITaVisitingLogService visitingLogService;
	@Autowired
	ITaDeviceService taDeviceService;
	
	@Autowired
	private ISysParamService iSysParamService;

	@Autowired
	private ITaVisitingWordsService visitingWordsService;

	@Autowired
	private ITaCustomWordsService taCustomWordsService;
	
	@Autowired
	private ITaPersonTypeService taPersonTypeService;

	@Autowired
	private ITaCustomerService iTaCustomerService;

	@Autowired
	private ITaCarRecordService iTaCarRecordService;

	@Autowired
	private XFYun xfYun;
	
	@Value("${huiju.files}")
	private String huijuFiles;

	@Autowired
	private ICameraService cameraService;

	@GetMapping("/ping")
	public String ping() {
		return "pong";
	}

	@ApiOperation(value = "人脸对比结果上传", notes = "人脸对比结果上传")
	@PostMapping(value = "/camera/{deviceId}/person")
	public ResponseBean person(
			@PathVariable String deviceId,
			MultipartFile snapshot,
			TaSnapshotPerson taSnapshotPerson,
			HttpServletRequest request) {
		ResponseBean responseBean = new ResponseBean();

		if (null == deviceId || null == taSnapshotPerson) {
			responseBean.addError(HttpStatus.SC_BAD_REQUEST, "设备ID 或者 抓拍参数传入方式不正确");
			return responseBean;
		}

		if (!deviceId.equals(taSnapshotPerson.getDeviceId())) {
			responseBean.addError(HttpStatus.SC_BAD_REQUEST, "设备ID 或者 抓拍参数传入不正确");
			return responseBean;
		}

		// 抓拍时间为当前时间
		taSnapshotPerson.setCreateDate(LocalDateTime.now());

		// 人像质量最低值, 低于这个值的认为人脸抓拍失败, 取值范围 1 - 100
		Float faceMinScore = iSysParamService.getFaceMinScore();

		Integer score = taSnapshotPerson.getScore();
		if (null == score || score.intValue() < faceMinScore) {
			return responseBean;
		}

		// 人脸相似度阈值, 低于设定认为没有识别成功
		// SDK 中这个值范围是 0.001 - 1, 本系统设定范围是 1 - 100
		Float samePersonThreshold = iSysParamService.getSamePersonThreshold();
		Float similarity = taSnapshotPerson.getSimilarity();

		// 识别结果
		boolean newGuy = null == similarity || similarity < (samePersonThreshold / 100);

		TaPerson taPerson = null;
		if (!newGuy) {
			taPerson = taPersonService.getById(taSnapshotPerson.getMatchPerson());

			// 抓拍人与比对人是同一个人
			taSnapshotPerson.setPersonId(taSnapshotPerson.getMatchPerson());

			LocalDateTime lastTime = taPerson.getLastVisiteDate();
			boolean sameDevice = deviceId.equals(taPerson.getLastVisiteDevice());

			if (null != lastTime && sameDevice) {
				// 降噪设定
				Float dispatchTimeLimit = iSysParamService.getDispatchTimeLimit();

				// 默认 2 分钟内连续识别的都是垃圾数据
				Duration duration = Duration.between(lastTime, taSnapshotPerson.getCreateDate());
				if (duration.getSeconds() < (60 * dispatchTimeLimit)) {
					return responseBean;
				}
			}
		}

		// 抓拍照
		taSnapshotPerson.setAvatar(getMultipartFile(snapshot));

		if (newGuy) {
			// 新人入库, 同时写入抓拍人ID
			taPerson = saveNewPerson(taSnapshotPerson);

			// 同步相机识别库
			cameraService.dispatchPerson(taPerson);
		}

		// 保存抓拍日志
		taSnapshotPersonService.save(taSnapshotPerson);

		// 预写到访日志
		saveVisitingLog(taPerson, taSnapshotPerson);

		// 判断是否客户来访
		try {
			iTaCustomerService.judgeNewCust(taPerson);
		} catch (Exception e) {
			log.error(e.getMessage());
		}

		// 推送消息
		sendPersionInfo(taPerson);

		return responseBean;
	}

	@PostMapping(value = "/camera/{deviceId}/car")
	public ResponseBean car(
			@PathParam("deviceId") String deviceId,
			@RequestParam("sceneImage") MultipartFile carImage,
			@RequestParam("licenseImage") MultipartFile licenseImage,
			@RequestParam(value = "driverImage", required = false) MultipartFile driverImage,
			@RequestParam(value = "copilotImage", required = false) MultipartFile copilotImage,
			@RequestParam("plateNumber") String plateNumber,
			@RequestParam("color") String color,
			@RequestParam("carModel") String carModel
			) {

		TaCarRecord taCarRecord = new TaCarRecord();
		taCarRecord.setPlateNumber(plateNumber);
		taCarRecord.setColor(color);
		taCarRecord.setCarModel(carModel);

		if (null != carImage && !carImage.isEmpty()) {
			taCarRecord.setCarImage(getMultipartFile(carImage));
		}
		if (null != licenseImage && !licenseImage.isEmpty()) {
			taCarRecord.setLicenseImage(getMultipartFile(licenseImage));
		}
		if (null != driverImage && !driverImage.isEmpty()) {
			taCarRecord.setDriverImage(getMultipartFile(driverImage));
		}
		if (null != copilotImage && !copilotImage.isEmpty()) {
			taCarRecord.setCopilotImage(getMultipartFile(copilotImage));
		}

		iTaCarRecordService.newRecordByDevice(taCarRecord);

		return ResponseBean.success("");
	}

	//推送给前端
	private void sendPersionInfo(TaPerson taPerson) {
		TaVisitingWords taVisitingWords = null;
		String deviceId = taPerson.getLastVisiteDevice();
		String words = taPerson.getWords();
		Integer sex = "2".equals(taPerson.getSex()) ? 2 : 1;
		String personName = taPerson.getName();
		if (null == personName) personName = "";

		// 检查是不是有定制语音
		QueryWrapper<TaCustomWords> wrapper = new QueryWrapper<>();
		wrapper.eq("device_id", deviceId);
		wrapper.eq("person_id", taPerson.getPersonId());
		TaCustomWords taCustomWords = taCustomWordsService.getOne(wrapper);
		if (null != taCustomWords) {
			String wds = taCustomWords.getWords();
			if (null != wds && !"".equals(wds.trim())) {
				words = wds.trim();
			}
		}

		if (null == words || "".equals(words)) {
			// 当前分类下不一定维护了欢迎词
			QueryWrapper<TaVisitingWords> query = new QueryWrapper<>();
			query.eq("device_id", deviceId);
			query.eq("type_id", taPerson.getTypeId());
			taVisitingWords = visitingWordsService.getOne(query);
			if (null == taVisitingWords) {
				return;
			}

			words = taVisitingWords.getWords();
		}


		words = words.replace("{{name}}", personName).replace("{{sex}}", sex == 2 ? "女士" : "先生");
		taVisitingWords.setWords(words);

		Map<String,Object> data = new HashMap<>();
		data.put("person",taPerson);
		data.put("taVisitingWords",taVisitingWords);
		data.put("audio", xfYun.voice( personName + "," + words, sex  == 2 ? 1 : 2));


		Map<String, Object> payload = new HashMap<>();
		payload.put("to", deviceId);
		payload.put("data", data);
		Action act = Action.create(Action.TYPE_TO_FRONT, payload);

		try {
			WebSocketServer.sendInfo(act.toJSON(), "device", deviceId);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	// 获取上传单文件
	private String getMultipartFile(MultipartFile f) {
		try {
			return AliOSSUtils.putObject(f);
		} catch (IOException e) {
			e.printStackTrace();
			log.error("上传人脸至阿里云失败: " + e.getMessage());
			return null;
		}
	}

	private TaPerson saveNewPerson(TaSnapshotPerson taSnapshotPerson) {
		TaPerson person = new TaPerson();
		person.setTypeId(NEW_PERSON);
		person.setAvatar(taSnapshotPerson.getAvatar());
		person.setStatus(StatusUtils.Normal);
		person.setLastVisiteDate(taSnapshotPerson.getCreateDate());
		person.setLastVisiteDevice(taSnapshotPerson.getDeviceId());
		person.setSex(taSnapshotPerson.getSex().toString());

		boolean rtn = taPersonService.save(person);
		if (!rtn) {
			return null;
		}

		// 更新 real_id
		UpdateWrapper<TaPerson> wrapper = new UpdateWrapper<>();
		wrapper.set("real_id", person.getPersonId());
		wrapper.eq("person_id", person.getPersonId());
		taPersonService.update(new TaPerson(), wrapper);

		taSnapshotPerson.setPersonId(person.getPersonId());
		person.setRealId(person.getPersonId());
		return person;
	}

	//
	private TaVisitingLog saveVisitingLog(TaPerson taPerson, TaSnapshotPerson taSnapshotPerson) {
		TaVisitingLog taVisitingLog = new TaVisitingLog();
		TaDevice taDevice = taDeviceService.getById(taSnapshotPerson.getDeviceId());

		taVisitingLog.setPersonId(taPerson.getPersonId());
		taVisitingLog.setRealPerson(taPerson.getRealId());
		taVisitingLog.setAvatar(taPerson.getAvatar());
		taVisitingLog.setDeviceId(taDevice.getDeviceId());
		taVisitingLog.setDeviceName(taDevice.getDeviceName());
		taVisitingLog.setVisiteDate(LocalDateTime.now());
		taVisitingLog.setName(taPerson.getName());

		// 反更新最后来访时间
		taPerson.setLastVisiteDate(taVisitingLog.getVisiteDate());
		taPerson.setLastVisiteDevice(taDevice.getDeviceId());
		UpdateWrapper<TaPerson> wrapper = new UpdateWrapper<>();
		wrapper.set("last_visite_date", taVisitingLog.getVisiteDate());
		wrapper.set("last_visite_device", taDevice.getDeviceId());
		wrapper.eq("person_id", taPerson.getPersonId());
		taPersonService.update(taPerson, wrapper);

		boolean rtn = visitingLogService.save(taVisitingLog);
		return !rtn ? null : taVisitingLog;
	}
}
