package com.huiju.welcome.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.serializer.SerializerFeature;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huiju.welcome.common.Action;
import com.huiju.welcome.common.WebSocketServer;
import com.huiju.welcome.mapper.TaFirstUsherRecordMapper;
import com.huiju.welcome.model.TaCarRecord;
import com.huiju.welcome.model.TaCustomer;
import com.huiju.welcome.model.TaFirstUsherRecord;
import com.huiju.welcome.service.ITaCustomerService;
import com.huiju.welcome.service.ITaFirstUsherRecordService;
import com.huiju.welcome.utils.CommConstant;
import com.huiju.welcome.utils.StatusUtils;
import com.huiju.welcome.utils.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author jobob
 * @since 2019-06-04
 */
@Service
public class TaFirstUsherRecordServiceImpl extends ServiceImpl<TaFirstUsherRecordMapper, TaFirstUsherRecord> implements ITaFirstUsherRecordService {
    @Autowired
    TaFirstUsherRecordMapper taFirstUsherRecordMapper;

    @Autowired
    ITaCustomerService iTaCustomerService;

    @Override
    public boolean updateNormal(List<TaFirstUsherRecord> records) {
        if (records == null || records.size() == 0) return true;

        List<Integer> recs = new ArrayList<>();
        for (TaFirstUsherRecord rec: records) {
            recs.add(rec.getRecId());
        }

        UpdateWrapper<TaFirstUsherRecord> wrapper = new UpdateWrapper();
        wrapper.set("status", StatusUtils.Normal);
        wrapper.in("rec_id", recs);

        int row = taFirstUsherRecordMapper.update(new TaFirstUsherRecord(), wrapper);
        return row >= 0;
    }

    @Override
    public TaFirstUsherRecord newRecordByDevice(TaCarRecord taCarRecord) {
        TaFirstUsherRecord taFirstUsherRecord = new TaFirstUsherRecord();
        taFirstUsherRecord.setStatus(StatusUtils.Ready);
        taFirstUsherRecord.setCarImage(taCarRecord.getCarImage());
        taFirstUsherRecord.setCarModel(taCarRecord.getCarModel());
        taFirstUsherRecord.setColor(taCarRecord.getColor());
        taFirstUsherRecord.setCarRecord(taCarRecord.getSerialNo());
        taFirstUsherRecord.setPlateNumber(taCarRecord.getPlateNumber());
        taFirstUsherRecord.setVisiteDate(LocalDateTime.now());

        taFirstUsherRecordMapper.insert(taFirstUsherRecord);

        return taFirstUsherRecord;
    }

    @Override
    public void notifyMiniapp(TaFirstUsherRecord taFirstUsherRecord) {
        Action act = Action.create(Action.TYPE_NEW_FIRST_USHER, JSONObject.toJSONString(taFirstUsherRecord, SerializerFeature.WriteMapNullValue));
        try {
            WebSocketServer.sendInfo(act.toJSON(), CommConstant.WECHAT_CLIENT_MINIAPP, null);
        } catch (Exception e) {}
    }

    @Override
    public boolean newRecordByMiniapp(TaFirstUsherRecord taFirstUsherRecord) {
        notifyConsultant(taFirstUsherRecord);
        int row = taFirstUsherRecordMapper.insert(taFirstUsherRecord);

        return row > 0;
    }

    @Override
    public boolean updateByMiniapp(TaFirstUsherRecord taFirstUsherRecord) {
        notifyConsultant(taFirstUsherRecord);
        int row = taFirstUsherRecordMapper.updateById(taFirstUsherRecord);
        return row > 0;
    }

    private void notifyConsultant(TaFirstUsherRecord taFirstUsherRecord) {
        String plateNumber = taFirstUsherRecord.getPlateNumber();

        // 依据车牌判断是否老客户来访
        if (!StringUtils.isEmpty(plateNumber)) {
            List<TaCustomer> taCustomerList = iTaCustomerService.getSomeCustMayBe(plateNumber);

            // 一个车牌对应多个客户, 先随机找一条数据
            if (null != taCustomerList && taCustomerList.size() > 0) {
                TaCustomer taCustomer = taCustomerList.get(0);
                try {
                    iTaCustomerService.notifyConsultant(taCustomer);
                } catch (Exception e) {}
            }
        }
    }

    @Override
    public IPage<TaFirstUsherRecord> taFirstUsherRecordlist(IPage pg,String plateNumber, String visiteDate) {
        IPage<TaFirstUsherRecord> pageTaFirstUsherRecord= taFirstUsherRecordMapper.taFirstUsherRecordlist(pg,plateNumber,visiteDate);
        return pageTaFirstUsherRecord;
    }
}
