package com.huiju.welcome.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huiju.welcome.mapper.TaDeviceMapper;
import com.huiju.welcome.mapper.TaPersonMapper;
import com.huiju.welcome.mapper.TaPersonTypeMapper;
import com.huiju.welcome.model.*;
import com.huiju.welcome.service.ICameraService;
import com.huiju.welcome.service.ITaOperationLogService;
import com.huiju.welcome.service.ITaPersonService;
import com.huiju.welcome.service.ITaVisitingLogService;
import com.huiju.welcome.utils.LogBuilder;
import com.huiju.welcome.utils.StatusUtils;
import com.huiju.welcome.utils.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.client.RestTemplate;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.time.LocalDateTime;
import java.util.*;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author jobob
 * @since 2019-03-12
 */
@Service
@Transactional(rollbackFor = Exception.class)
public class TaPersonServiceImpl extends ServiceImpl<TaPersonMapper, TaPerson> implements ITaPersonService {

    private Logger log = LoggerFactory.getLogger(TaPersonServiceImpl.class);

    @Autowired
    private TaPersonMapper taPersonMapper;

    @Autowired
    private TaPersonTypeMapper taPersonTypeMapper;

    @Autowired
    private ITaVisitingLogService taVisitingLogService;
    
    @Autowired
    private TaDeviceMapper taDeviceMapper;

    @Autowired
    private ITaOperationLogService taOperationLogService;

    @Autowired
    private ICameraService iCameraService;
    
    @Value("${huiju.files}")
    private String huijuFiles;

    @Override
    public String updateAvatar(TaPerson taPerson, TaDevice ...dev) {
        String state = "";

        List<TaDevice> taDeviceList = null;
        if (null == dev || dev.length == 0) {
            ByteArrayOutputStream bos = new ByteArrayOutputStream(1000);
            QueryWrapper<TaDevice> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("status", StatusUtils.Normal);
            queryWrapper.eq("afr", StatusUtils.BoolTrue);
            taDeviceList = taDeviceMapper.selectList(queryWrapper);
        } else {
            taDeviceList = Arrays.asList(dev);
        }

        try {
            //分发人脸到各个相机
            RestTemplate restTemplate = new RestTemplate();
            //获取文件流
            String avatar = taPerson.getAvatar();
            if (avatar.startsWith("/")) {
                avatar = avatar.substring(1);
            }

            FileInputStream fileInputStream = new FileInputStream(new File(huijuFiles + avatar));
            byte[] byt = new byte[fileInputStream.available()];
            fileInputStream.read(byt);

            for(TaDevice taDevice : taDeviceList){
                String cameraUrl = taDevice.getApi() + "/video/" + taPerson.getPersonId();
                HttpHeaders headers = new HttpHeaders();
                headers.setContentType(MediaType.APPLICATION_OCTET_STREAM);
                HttpEntity httpEntity = new HttpEntity(byt);
                String result = restTemplate.exchange(cameraUrl, HttpMethod.POST, httpEntity, String.class).getBody();
                JSONObject jsonObject = JSONObject.parseObject(result);
                if (!jsonObject.get("code").equals(200)){
                    log.error(" 下发人脸失败: "+  jsonObject.get("message").toString());
                    return taDevice.getDeviceName() + "请检查状态！";
                }
            }
        }catch (Exception e){
            e.printStackTrace();
            return "更新头像失败";
        }
        boolean status = taPersonMapper.updatePersonAvatar(taPerson);
        if (status){
            return "success";
        }
        return "更新头像失败";
    }

    @Override
    public Map<String, Object> detail(int id) {
        TaPerson taPerson = getById(id);

        QueryWrapper<TaVisitingLog> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("real_person", taPerson.getRealId());
        queryWrapper.orderByDesc("visite_date");

        List<TaVisitingLog> logs = taVisitingLogService.list(queryWrapper);

        Map<String, Object> res = new HashMap<>();
        res.put("detail", taPerson);
        res.put("visitingLog", logs);
        return res;
    }
    
    /**
     * 获取详细信息
     * @param personId
     * @param deviceId
     * @return
     */
    @Override
    public Map<String, String> getDetailInfo(Integer personId, String deviceId) {
        return taPersonMapper.getDetailInfo(personId,deviceId);
    }

    @Override
    public List<Map<String, String>> statsPerson() {
        return taPersonMapper.statsPerson();
    }

    @Override
    public boolean updateSales(TaPerson taPerson) {
        UpdateWrapper<TaPerson> wrapper = new UpdateWrapper<>();
        wrapper.eq("person_id", taPerson.getPersonId());
        wrapper.isNull("last_sales");

        return taPersonMapper.update(taPerson, wrapper) >= 0;
    }

    @Override
    public boolean mergePerson(Integer fromPerson, TaPerson toPerson) {
        // 合并基础信息
        UpdateWrapper<TaPerson> wrapper = new UpdateWrapper<>();
        wrapper.set("name", toPerson.getName());
        wrapper.set("sex", toPerson.getSex());
        wrapper.set("type_id", toPerson.getTypeId());
        wrapper.set("phone", toPerson.getPhone());
        wrapper.set("email", toPerson.getEmail());
        wrapper.set("remark", toPerson.getRemark());
        wrapper.set("real_id", toPerson.getRealId());
        wrapper.set("words", toPerson.getWords());
        wrapper.eq("person_id", fromPerson);

        boolean success = taPersonMapper.update(new TaPerson(), wrapper) >= 0;
        if (!success) return false;

        wrapper.eq("person_id", toPerson.getPersonId());
        success = taPersonMapper.update(new TaPerson(), wrapper) >= 0;
        if (!success) return false;

        // 合并访问记录
        TaVisitingLog taVisitingLog = new TaVisitingLog();
        taVisitingLog.setRealPerson(toPerson.getRealId());
        taVisitingLog.setPersonId(fromPerson);
        UpdateWrapper<TaVisitingLog> wrapper2 = new UpdateWrapper<>();
        wrapper2.eq("person_id", fromPerson);
        return taVisitingLogService.update(taVisitingLog, wrapper2);
    }

    @Override
    public IPage<TaPerson> getRealPersonList(Page<TaPerson> page, Integer personId, Integer typeId, String name, String phone, String beginDate, String endDate) {
        return taPersonMapper.getRealPersonList(page, personId, typeId, name, phone, beginDate, endDate);
    }

    @Override
    public void logNew(String opertaor, TaPerson taPerson) {
        String source = "人员信息";
        String optType = "新增";
        String entity = taPerson.getName();

        LogBuilder logBuilder = new LogBuilder();
        logBuilder.append("姓名", taPerson.getName());
        logBuilder.append("类型", getTypeName(taPerson));
        logBuilder.append("性别", getSex(taPerson));
        logBuilder.append("头像", taPerson.getAvatar());
        logBuilder.append("手机", taPerson.getPhone());
        logBuilder.append("邮箱", taPerson.getEmail());
        logBuilder.append("备注", taPerson.getRemark());
        logBuilder.append("欢迎词", taPerson.getWords());

        String content = logBuilder.build();
        taOperationLogService.create(opertaor, source, optType, entity, String.valueOf(taPerson.getPersonId()), content);
    }

    @Override
    public void logEdit(String opertaor, TaPerson from, TaPerson to) {
        String source = "人员信息";
        String optType = "编辑";
        String entity = from.getName();

        LogBuilder logBuilder = new LogBuilder();
        logBuilder.append("姓名", from.getName(), to.getName());
        logBuilder.append("类型", getTypeName(from), getTypeName(to));
        logBuilder.append("性别", getSex(from), getSex(to));
        logBuilder.append("头像", from.getAvatar(), to.getAvatar());
        logBuilder.append("手机", from.getPhone(), to.getPhone());
        logBuilder.append("邮箱", from.getEmail(), to.getEmail());
        logBuilder.append("备注", from.getRemark(), to.getRemark());
        logBuilder.append("欢迎词", from.getWords(), to.getWords());

        String content = logBuilder.build();
        taOperationLogService.create(opertaor, source, optType, entity, String.valueOf(from.getPersonId()), content);
    }

    @Override
    public TaPerson getByPhone(String phone) {
        QueryWrapper<TaPerson> wrapper = new QueryWrapper<>();
        wrapper.eq("phone", phone);
        wrapper.eq("status", StatusUtils.Normal);
        wrapper.last("limit 1");

        return taPersonMapper.selectOne(wrapper);
    }

    @Override
    public TaPerson newPersonByFront(TaCustomer taCustomer) {
        TaPerson taPerson = new TaPerson();
        taPerson.setAvatar(taCustomer.getAvatar());
        taPerson.setPhone(taCustomer.getPhone());
        taPerson.setName(taCustomer.getCustomerName());
        taPerson.setStatus(StatusUtils.Normal);
        taPerson.setCreateDate(LocalDateTime.now());

        // 直接是VIP
        taPerson.setTypeId(2);
        int row = taPersonMapper.insert(taPerson);
        if (row < 0) return null;

        taPerson.setRealId(taPerson.getPersonId());
        taPersonMapper.updateById(taPerson);

        // 下发人脸到各设备
        iCameraService.dispatchPerson(taPerson);

        return taPerson;
    }

    @Override
    public TaPerson newPersonByUsher(TaMainUsherRecord taMainUsherRecord) {
        TaPerson taPerson = new TaPerson();
        taPerson.setPhone(taMainUsherRecord.getPhone());
        taPerson.setName(taMainUsherRecord.getPersonName());
        taPerson.setStatus(StatusUtils.Normal);
        taPerson.setCreateDate(LocalDateTime.now());

        // 直接是VIP
        taPerson.setTypeId(2);
        int row = taPersonMapper.insert(taPerson);
        if (row < 0) return null;

        taPerson.setRealId(taPerson.getPersonId());
        taPersonMapper.updateById(taPerson);

        return taPerson;
    }

    @Override
    public boolean updateByCustomer(TaCustomer taCustomer) {
        if (StringUtils.isEmpty(taCustomer.getPhone())) return true;

        // 默认客户与人员ID是一致的
        UpdateWrapper<TaPerson> updateWrapper = new UpdateWrapper<>();
        updateWrapper.set("phone", taCustomer.getPhone());
        updateWrapper.set("name", taCustomer.getCustomerName());
        updateWrapper.eq("real_id", taCustomer.getCustomerId()).or().eq("person_id", taCustomer.getCustomerId());

        int row = taPersonMapper.update(new TaPerson(), updateWrapper);
        return row > 0;
    }

    /**
     * 加入白名单， 实际上置为工作人员
     * @param personId
     * @return
     */
    @Override
    public boolean setPersonIntoWhitelist(Integer personId) {

        UpdateWrapper<TaPerson> updateWrapper = new UpdateWrapper<>();
        updateWrapper.set("type_id", 3);
        updateWrapper.eq("person_id", personId);

        int row = taPersonMapper.update(new TaPerson(), updateWrapper);

        return row > 0;
    }

    private String getTypeName(TaPerson taPerson) {
        TaPersonType taPersonType = taPersonTypeMapper.selectById(taPerson.getTypeId());
        return null == taPersonType ? null : taPersonType.getTypeName();
    }

    private String getSex(TaPerson taPerson) {
        if (null == taPerson.getSex()) return null;

        Map<String, String> dict = new HashMap<>();
        dict.put("1", "男");
        dict.put("1", "女");
        dict.put("9", "未知");

        String sex = dict.get(taPerson.getSex());
        return null == sex ? taPerson.getSex() : sex;
    }
}
