package com.huiju.welcome.utils;

import com.alibaba.fastjson.JSONObject;
import com.aliyuncs.CommonRequest;
import com.aliyuncs.CommonResponse;
import com.aliyuncs.DefaultAcsClient;
import com.aliyuncs.http.MethodType;
import com.aliyuncs.profile.DefaultProfile;
import com.huiju.welcome.config.AliProperties;
import lombok.extern.slf4j.Slf4j;
import org.apache.http.HttpStatus;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.context.properties.EnableConfigurationProperties;
import org.springframework.context.annotation.Configuration;

@Slf4j
@Configuration
@EnableConfigurationProperties(AliProperties.class)
public class AliFaceUtils {

    public final static String ACT_ADD_FACE = "AddFace";
    public final static String ACT_RECOGNIZE_FACE = "RecognizeFace";

    DefaultProfile profile;
    DefaultAcsClient client;

    @Autowired
    public AliFaceUtils(AliProperties p) {
        profile = DefaultProfile.getProfile(AliProperties.face.getRegionId(), AliProperties.accessKeyId, AliProperties.accessKeySecret);
        client = new DefaultAcsClient(profile);
    }

    /**
     * 添加人脸
     * @param personId
     * @param name
     * @param url
     * @param datas
     * @throws Exception
     */
    public void addFace(Integer personId, String name, String url, String ...datas) throws Exception {
        log.info("阿里人脸比对, 请求上传人脸: {}[{}] {}", name, personId, url);

        if (null == url && null == datas) {
            throw new Exception("图片地址 或者 Base64 数据必填一个");
        }

        CommonRequest request = generalRequest(ACT_ADD_FACE);
        request.putBodyParameter("Person", name);
        request.putBodyParameter("Image", personId);

        if (null == datas || datas.length < 1) {
            request.putBodyParameter("ImageUrl", url);
        } else {
            request.putBodyParameter("Content", datas[0]);
        }

        excuteRequest(request);
    }

    /**
     * 人脸库比对
     * @param url
     * @param datas
     * @throws Exception
     */
    public JSONObject recognizeFace(String url, String ...datas) throws Exception {
        log.info("阿里人脸比对, 请求比对人脸: {}", url);

        if (null == url && null == datas) {
            throw new Exception("图片地址 或者 Base64 数据必填一个");
        }

        CommonRequest request = generalRequest(ACT_RECOGNIZE_FACE);

        if (null == datas || datas.length < 1) {
            request.putBodyParameter("ImageUrl", url);
        } else {
            request.putBodyParameter("Content", datas[0]);
        }

        return excuteRequest(request);
    }

    private JSONObject excuteRequest(CommonRequest request) throws Exception {
        CommonResponse response = client.getCommonResponse(request);
        if (response.getHttpStatus() == HttpStatus.SC_OK) {
            try {
                JSONObject result = JSONObject.parseObject(response.getData());

                Boolean success = result.getBoolean("Success");
                if (null == success || !success) {
                    String error = "校验阿里人脸比对接口返回出错: " + result.getString("Message");
                    log.error(error);
                    throw new Exception(error);
                }

                return result;
            } catch (Exception e) {
                String error = "校验阿里人脸比对接口出错: " + e.getMessage();
                log.error(error);
                e.printStackTrace();
                throw new Exception(error);
            }
        } else {
            String error = "网络请求上传比对人脸失败, 错误码: {}" + response.getHttpStatus();
            log.error(error);
            throw new Exception(error);
        }
    }

    private CommonRequest generalRequest(String action) {
        CommonRequest request = new CommonRequest();
        request.setMethod(MethodType.POST);
        request.setDomain(AliProperties.face.getDomain());
        request.setVersion(AliProperties.face.getVersion());
        request.setAction(action);
        request.putBodyParameter("Group", AliProperties.face.getGroup());
        return request;
    }
}
